--[[

TidyShop: Buy Used Equipment - Adds the option to buy vehicles & implements as used/second hand

Author:     w33zl / WZL Modding
Version:    1.0.1
Modified:   2020-11-12

Changelog:
v1.0        Initial public release
v0.9.2      Tweaked both vanilla & seasons
v0.9.1      Seasons support
v0.9        Private beta

]]

UsedEquipment = Mod:init()

local CONFIGTYPE_NAME = "vehicleBuyCondition"
local CONFIG_NEW_NAME = "new"
local CONFIG_USED_NAME = "used"

function UsedEquipment:registerChangedShopItem(shopItem)

    self.touchedShopItems = self.touchedShopItems or {}
    self.touchedShopItems.byID = self.touchedShopItems.byID or {}
    self.touchedShopItems.byFilename = self.touchedShopItems.byFilename or {}

    self.touchedShopItems.byID[shopItem.id] = shopItem
    self.touchedShopItems.byFilename[shopItem.xmlFilenameLower] = shopItem

    self.lastShopItem = shopItem
end
function UsedEquipment:unRegisterChangedShopItem(shopItem)
    self.lastShopItem = nil
end

function UsedEquipment:resetLastShopItem()
    local lastShopItem = self.lastShopItem

    if lastShopItem == nil then return end

    if lastShopItem.originalPrice ~= nil then
        if lastShopItem.originalPrice ~= lastShopItem.price then
            UsedEquipment:printDebug("resetLastShopItem: Resetting price")
            lastShopItem.price = lastShopItem.originalPrice
            lastShopItem.originalPrice = nil
        else
            UsedEquipment:printDebug("resetLastShopItem: No need to reset price")
        end
    end

    if lastShopItem.configurations ~= nil then
        local vehicleConditionConfig = lastShopItem.configurations[CONFIGTYPE_NAME]

        if vehicleConditionConfig ~= nil then
            lastShopItem.configurations[CONFIGTYPE_NAME] = UsedEquipment:getDefaultConfigurations()
        end
    end

    UsedEquipment:unRegisterChangedShopItem(lastShopItem)
end

ShopItemsFrame.onFrameOpen = Utils.prependedFunction(ShopItemsFrame.onFrameOpen, function()
    UsedEquipment:resetLastShopItem()
end)

ShopCategoriesFrame.onFrameClose = Utils.prependedFunction(ShopCategoriesFrame.onFrameClose, function()
    UsedEquipment:resetLastShopItem()
end)

local function addConfigOption(name, price, index, isDefault)
    return {
        name = g_i18n:getText("configuration_" .. name),
        index = index or 1,
        isDefault = isDefault or false,
        price = price or 0,
        dailyUpkeep = 0,
        desc = ""
    }
end

function UsedEquipment:getDefaultConfigurations()
    self.defaultConfigurations = self.defaultConfigurations or  {
        addConfigOption(CONFIG_USED_NAME, 0, 1, true)
    }

    return self.defaultConfigurations
end

function UsedEquipment:canBeBoughtUsed(storeItem)

    if storeItem == nil then return false end


    if storeItem.canBeBoughtUsed == nil then
        storeItem.canBeBoughtUsed = false

        local species = storeItem.species
        
        if species ~= nil and type(species) == "string" then
            species = species:upper()
        end

        if species == "PLACEABLE" or species == "HANDTOOL" then 
            return storeItem.canBeBoughtUsed
        end

        TidyShopHelper:extendShopItem(storeItem)

        storeItem.canBeBoughtUsed = storeItem.canBeBoughtUsed or storeItem:query():isCategoryType(StoreManager.CATEGORY_TYPE.VEHICLE):matches()
        storeItem.canBeBoughtUsed = storeItem.canBeBoughtUsed or storeItem:query():isCategoryType(StoreManager.CATEGORY_TYPE.TOOL):matches()
    end

    return storeItem.canBeBoughtUsed
end

local CONDITION_TIERS = {
    [1] = { discount = 0,       opTimeFactor = 0,       opTimeFactorTool = 0,       wearFactor = 0,     wearFactorTool = 0,     age = 0,    ageTool = 0 }, -- NEW
    [2] = { discount = 0.32,    opTimeFactor = 0.2,     opTimeFactorTool = 0.061,   wearFactor = 0.7,   wearFactorTool = 0.7,   age = 2,    ageTool = 3 }, -- USED T1
}

-- Buy event (override hook)
ShopController.buy = Utils.overwrittenFunction(ShopController.buy, function(self, superFunc, shopItem, ...)

    if shopItem.configurations ~= nil and UsedEquipment:canBeBoughtUsed(shopItem) then
        local vehicleConditionConfig = shopItem.configurations[CONFIGTYPE_NAME]

        if vehicleConditionConfig ~= nil then

            local seasons = g_seasons or getfenv(0)["g_seasons"]
            local currentTier = CONDITION_TIERS[2]

            shopItem.originalPrice = shopItem.originalPrice or shopItem.price -- fallback if previous reset has failed

            UsedEquipment:printDebugVar("buy.seasons", seasons)

            if seasons ~= nil then
                UsedEquipment:printDebug("buy:: seasons is active")
                shopItem.usedVehicleDiscount = -math.floor( shopItem.originalPrice * currentTier.discount )
            else
                shopItem.usedVehicleDiscount = -math.floor( shopItem.originalPrice * 0.2 )
            end

            shopItem.configurations[CONFIGTYPE_NAME] = {
                addConfigOption(CONFIG_NEW_NAME, 0, 1, true),
                addConfigOption(CONFIG_USED_NAME, shopItem.usedVehicleDiscount, 2, false),
            }

            UsedEquipment:registerChangedShopItem(shopItem)

        end
    end

    return superFunc(self, shopItem, ...)
end)--BUY

-- addVehicle event (override hook)
FSBaseMission.addVehicle = Utils.overwrittenFunction(FSBaseMission.addVehicle, function(currentMission, superFunc, newVehicle)

    if newVehicle ~= nil then

        if newVehicle.configurations[CONFIGTYPE_NAME] ~= nil and newVehicle.configurations[CONFIGTYPE_NAME] > 1 then

            UsedEquipment:printDebugVar("newVehicle.price", newVehicle.price)
            UsedEquipment:printDebugVar("newVehicle.originalPrice", newVehicle.originalPrice)
            UsedEquipment:printDebugVar("UsedEquipment.lastShopItem", UsedEquipment.lastShopItem)

            UsedEquipment:printDebug("Setting wear and tear")

            local storeItem = nil
            local lifetime = nil or 600


            local optime = 18720000

            local currentTier = CONDITION_TIERS[2]
            local isMotorized = newVehicle.spec_motorized ~= nil

            -- local age = currentTier.age
            
            local wearFactor = (isMotorized and currentTier.wearFactor) or currentTier.wearFactorTool
            local isSeasons = (newVehicle["spec_FS19_RM_Seasons.seasonsVehicle"] ~= nil)
            local age = (isMotorized == false and isSeasons == true and currentTier.ageTool) or currentTier.age
            local opTimeFactor = (isMotorized == false and isSeasons == true and currentTier.opTimeFactorTool) or currentTier.opTimeFactor
            local dirtFactor = (isSeasons and 0.5) or 0.65

            UsedEquipment:printDebugVar("newVehicle.isMotorized", isMotorized)

            if isSeasons == true then
                local seasonsVehicle = newVehicle["spec_FS19_RM_Seasons.seasonsVehicle"]

                UsedEquipment:printDebug("Seasons found")

                if seasonsVehicle ~= nil then
                    seasonsVehicle.years = age
                    UsedEquipment:printDebugVar("seasonsVehicle.nextRepair", seasonsVehicle.nextRepair)
                    -- local opTimeFactor = (isMotorized and currentTier.opTimeFactor) or currentTier.opTimeFactorTool
                    local x = lifetime * opTimeFactor * 3600

                    UsedEquipment:printDebugVar("newVehicle.opTimeFactor", opTimeFactor)
                    -- newVehicle:setOperatingTime(x * 1000, true)
                    optime = x * 1000

                    seasonsVehicle.nextRepair = x + (108000 * (1 - wearFactor))
                end

                newVehicle.age = (g_seasons.environment.daysPerSeason or 9) * age
                -- newVehicle.price = newVehicle.originalPrice or (newVehicle.price * 1.25 * 1.25)
                newVehicle.price = newVehicle.price + math.abs( UsedEquipment.lastShopItem.usedVehicleDiscount)
            else
                newVehicle.age = age * 6
                -- newVehicle.price = newVehicle.price * 1.23
                newVehicle.price = newVehicle.price + math.abs( UsedEquipment.lastShopItem.usedVehicleDiscount)

                if newVehicle.spec_wearable ~= nil then
                    UsedEquipment:printDebug("Setting wear levels")
    
                    local spec = newVehicle.spec_wearable
                    for _, data in ipairs(spec.wearableNodes) do
                        newVehicle:setNodeWearAmount(data, 0.89, true)
                    end
                    newVehicle:raiseDirtyFlags(spec.dirtyFlag)
    
                end
            end

            if newVehicle.setOperatingTime ~= nil then
                newVehicle:setOperatingTime(optime, true)
            end

            if newVehicle.addDirtAmount ~= nil then
                newVehicle:addDirtAmount(dirtFactor)
                -- newVehicle:addDirtAmount(0.45)
            end


            if newVehicle.spec_fillUnit ~= nil then
                local fillUnits = newVehicle.spec_fillUnit.fillUnits

                if fillUnits ~= nil then
                    for _, fillUnit in ipairs(fillUnits) do
                        if fillUnit.fillType == g_fillTypeManager.nameToIndex["DIESEL"] then
                            fillUnit.fillLevel = fillUnit.capacity * 0.37
                        end
                    end
                end
            end

            newVehicle.configurations[CONFIGTYPE_NAME] = 1
        end
    end

    local x = superFunc(currentMission, newVehicle)

    return x
end)

-- Add new config type
g_configurationManager:addConfigurationType(CONFIGTYPE_NAME, g_i18n:getText("configuration_title"), nil, nil, nil, nil, ConfigurationUtil.SELECTOR_MULTIOPTION)

-- Injecty new config type to all machines via getConfigurationsFromXML event
StoreItemUtil.getConfigurationsFromXML = Utils.overwrittenFunction(StoreItemUtil.getConfigurationsFromXML, function(xmlFile, superFunc, baseXMLName, baseDir, customEnvironment, isMod, storeItem)
	local configurations = superFunc(xmlFile, baseXMLName, baseDir, customEnvironment, isMod, storeItem)
    -- UsedEquipment:printDebug("StoreItemUtil.getConfigurationsFromXML")

    -- UsedEquipment:printDebugVar("storeItem", storeItem)
    -- if storeItem ~= nil then
    --     UsedEquipment:printDebugVar("storeItem.species", storeItem.species)
    -- end
    
    -- if storeItem.species == "placeable" or storeItem.species == "handTool" then
        
    -- else

        if UsedEquipment:canBeBoughtUsed(storeItem) then
            configurations = configurations or {}

            configurations[CONFIGTYPE_NAME] = UsedEquipment:getDefaultConfigurations()
        end
    -- end

    return configurations
end)
